﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using SponsorRun.Serial;
using System.IO;

namespace SponsorRun {

  public partial class MainForm : Form {
    bool switchTimer = false;
    SerialPortManager _spManager;

    // /*
    const int timeToRun = 5400; // in seconds
    const int fraudeInterval = 55; // time in which lap normally can be run (55 is reasonable for 400m)
    const int intervalDoubleReg = 7; // avoid double registration
    // */
    string[] keys = new string[1]; // RFID's used
    long[] currentTimes = new long[1]; 
    long[] lapTimes = new long[1];
    long[] startTimes = new long[1]; // keeps the start time for each team
    string[] teamStatus = new string[1]; // remaining time
    string[] teams = new string[1]; // team names
    int[] laps = new int[1]; // number of laps
    int nextKey = 1;
    public MainForm() {
      InitializeComponent();

      UserInitialization();
    }


    private void UserInitialization() {
      _spManager = new SerialPortManager();
      SerialSettings mySerialSettings = _spManager.CurrentSerialSettings;
      serialSettingsBindingSource.DataSource = mySerialSettings;
      portNameComboBox.DataSource = mySerialSettings.PortNameCollection;
      baudRateComboBox.DataSource = mySerialSettings.BaudRateCollection;
      dataBitsComboBox.DataSource = mySerialSettings.DataBitsCollection;
      parityComboBox.DataSource = Enum.GetValues(typeof(System.IO.Ports.Parity));
      stopBitsComboBox.DataSource = Enum.GetValues(typeof(System.IO.Ports.StopBits));

      _spManager.NewSerialDataRecieved += new EventHandler<SerialDataEventArgs>(_spManager_NewSerialDataRecieved);
      this.FormClosing += new FormClosingEventHandler(MainForm_FormClosing);
    }


    private void MainForm_FormClosing(object sender, FormClosingEventArgs e) {
      _spManager.Dispose();
    }

    void _spManager_NewSerialDataRecieved(object sender, SerialDataEventArgs e) {
      if (this.InvokeRequired) {
        // Using this.Invoke causes deadlock when closing serial port, and BeginInvoke is good practice anyway.
        this.BeginInvoke(new EventHandler<SerialDataEventArgs>(_spManager_NewSerialDataRecieved), new object[] { sender, e });
        return;
      }

      int maxTextLength = 200; // maximum text length in text box
      if (tbData.TextLength > maxTextLength)
        tbData.Text = tbData.Text.Remove(0, tbData.TextLength - (maxTextLength - 50));
      if (tbFraude.TextLength > maxTextLength)
        tbFraude.Text = tbFraude.Text.Remove(0, tbFraude.TextLength - (maxTextLength - 50));

      // data received is converted to text
      string str = Encoding.ASCII.GetString(e.Data);
      Console.WriteLine(str.Length);
      if (str.Length == 10) {
        timer1.Enabled = true;
        tbData.ScrollToCaret();
        tbFraude.ScrollToCaret();
        if (rdoReadKeys.Checked) {
          saveKeyToFile(str);
        }
        if (rdoRegisterLap.Checked) {
          registerLap(str);
        }
      }

    }

    // Handles the "Start Listening"-buttom click event
    private void btnStart_Click(object sender, EventArgs e) {
      _spManager.StartListening();
      btnStop.Enabled = true;
      btnStart.Enabled = false;
    }

    // Handles the "Stop Listening"-buttom click event
    private void btnStop_Click(object sender, EventArgs e) {
      _spManager.StopListening();
      btnStart.Enabled = true;
      btnStop.Enabled = false;
    }

    private void MainForm_Load(object sender, EventArgs e) {
      readDataFromFiles();
      fillFormGrid();
    }

    private void toonStatus(object sender, EventArgs e) {
      if (!switchTimer) {
        lblStatus.BackColor = Color.LightGreen;
        lblStatus.Text = "Registered ...";
        timer1.Interval = 500;
      }
      else {
        lblStatus.BackColor = Color.Yellow;
        lblStatus.Text = "Waiting ...";
        timer1.Interval = 1;
        timer1.Enabled = false;
      }
      switchTimer = !switchTimer;
    }
    private void saveKeyToFile(string str) {
      if (!Array.Exists(keys, element => element == str)) {
        keys[nextKey - 1] = str;
        nextKey++;
        Array.Resize(ref keys, nextKey);
        StreamWriter streamOutput = new StreamWriter("rfidKeys.txt", true);
        streamOutput.Write(str);
        streamOutput.Close();
        streamOutput.Dispose();
      }
    }
    private void registerLap(string str) {
      str = str.Substring(0, 8); // Remove obsolete data after the key
      // Find the key in the array
      int arrayLocation = Array.FindIndex(keys, element => element == str);
      tbData.BackColor = Color.LightGray;
      long timeNow = DateTime.Now.Ticks / 10000000;
      if ((timeNow - currentTimes[arrayLocation]) < intervalDoubleReg) {
        Console.Beep(800, 500);
        tbData.AppendText("Double registration" + Environment.NewLine);
        tbData.ScrollToCaret();
        return;
      }
      else if ((timeNow - currentTimes[arrayLocation]) < fraudeInterval) {
        Console.Beep(800, 500);
        tbFraude.BackColor = Color.Red;
        tbFraude.AppendText(teams[arrayLocation] +" FRAUDE " + Environment.NewLine);
        currentTimes[arrayLocation] = DateTime.Now.Ticks / 10000000;
        tbFraude.ScrollToCaret();
        StreamWriter fraude = new StreamWriter("fraude.txt", true);
        string outputFraude = teams[arrayLocation] + ";" + laps[arrayLocation] + ";" +DateTime.Now.ToLongTimeString();
        fraude.WriteLine(outputFraude);
        fraude.Close();
        fraude.Dispose();
        return;
      }

      Console.Beep(2000, 500);
      tbData.AppendText("Team " + (arrayLocation + 1) + " > " + teams[arrayLocation] + Environment.NewLine);
      if (laps[arrayLocation] == 0) {
        startTimes[arrayLocation] = DateTime.Now.Ticks / 10000000;
        int hoursLeft = (int)timeToRun / 3600;
        int minutesLeft = ((int)timeToRun - (hoursLeft * 3600)) / 60  ;
        int secondsLeft = (int)timeToRun % 60;
        teamStatus[arrayLocation] = hoursLeft.ToString("D1") + ":" + minutesLeft.ToString("D2") + ":" + secondsLeft.ToString("D2");
      }
      else {
        long timeLeft = timeToRun -  (timeNow - startTimes[arrayLocation]);
        if (timeLeft <= 0) {
          teamStatus[arrayLocation] = "Finished !!!";
          fillFormGrid();
          return;
        }
        int hoursLeft = (int)timeLeft / 3600;
        int minutesLeft = ((int) timeLeft - (hoursLeft*3600)) / 60;
        int secondsLeft = (int)timeLeft % 60;
        teamStatus[arrayLocation] = hoursLeft.ToString("D1") + ":" + minutesLeft.ToString("D2") + ":" + secondsLeft.ToString("D2");
      }

      lapTimes[arrayLocation] = DateTime.Now.Ticks / 10000000 - currentTimes[arrayLocation];

      currentTimes[arrayLocation] = DateTime.Now.Ticks / 10000000;
      StreamWriter resultaat = new StreamWriter("results.txt", true);
      string schrijven = str + ";" + DateTime.Now.Ticks / 10000000 + ";" + laps[arrayLocation] + ";" + teams[arrayLocation] + ";" + currentTimes[arrayLocation] + ";" + DateTime.Now.ToLocalTime();
      resultaat.WriteLine(schrijven);
      resultaat.Close();
      resultaat.Dispose();
      // fill 
      fillFormGrid();
      laps[arrayLocation]++;

    }

    private void fillFormGrid() {
      dgvList.Rows.Clear();
      for (int i = 1; i < keys.Length; i++) {
        string[] row = new String[] { i.ToString("D2"), teams[i - 1], laps[i - 1].ToString("D5"), lapTimes[i - 1].ToString("D6"), teamStatus[i - 1] };
        dgvList.Rows.Add(row);
      }
    }

    private void readDataFromFiles() {
      // read keys from file rfidKeys.txt
      if (File.Exists("rfidKeys.txt")) {
        StreamReader streamInput = new StreamReader("rfidKeys.txt");
        while (streamInput.Peek() != -1) {
          string inputLine = streamInput.ReadLine();
          keys[nextKey - 1] = inputLine;
          nextKey++;
          Array.Resize(ref keys, nextKey);
        }
        Array.Resize(ref currentTimes, nextKey);
        Array.Resize(ref laps, nextKey);
        Array.Resize(ref lapTimes, nextKey);
        Array.Resize(ref startTimes, nextKey);
        Array.Resize(ref teamStatus, nextKey);
        Array.Resize(ref teams, nextKey);

        streamInput.Close();
        streamInput.Dispose();
      }
      int nummer = 1;
      if (File.Exists("teams.txt")) {
        StreamReader streamInput = new StreamReader("teams.txt");
        while (streamInput.Peek() != -1) {
          string inputLine = streamInput.ReadLine();
          // avoid having more teams than keys available
          if (nummer <= nextKey) {
            teams[nummer - 1] = inputLine;
            nummer++;
          }
        }
      }
      // read data from results.txt, if it exists
      // added to make a restart of the program possible
      if (File.Exists("results.txt")) {
        StreamReader streamResult = new StreamReader("results.txt");
        while (streamResult.Peek() != -1) {
          string inputLine = streamResult.ReadLine();
          string[] inputPart = inputLine.Split(';');
          string rfidKey = inputPart[0];
          string numberOfLap = inputPart[2];
          string startTime = inputPart[4];
          int arrayLocation = Array.FindIndex(keys, element => element == rfidKey);
          if (arrayLocation >= 0) {
            laps[arrayLocation]++;
          }
          if (numberOfLap == "0") {
            startTimes[arrayLocation] = Convert.ToInt64(startTime);
          }
          long timeLeft = timeToRun - ((DateTime.Now.Ticks / 10000000) - startTimes[arrayLocation]);
          if (timeLeft <= 0) {
            teamStatus[arrayLocation] = "Finished !!!";
          }
          else {
            int hoursLeft = (int)timeLeft / 3600;
            int minutesLeft = ((int)timeLeft - (hoursLeft * 3600)) / 60 ;
            int secondsLeft = (int)timeLeft % 60;
            teamStatus[arrayLocation] = hoursLeft.ToString("D1") + ":" + minutesLeft.ToString("D2") + ":" + secondsLeft.ToString("D2");

          }
        }
        streamResult.Close();
        streamResult.Dispose();
      }
    }
  }
}
